/*START*/
/*TITLE calc_sigmas() */
/*
Source File :  /usr/ring_profile/ringpro_fit/src/calc_sigmas.c
Author(s) :  rgf
Latest Revision Date :  8/28/86
Description :  calculate sigma's based on an input square matrix
Algorithm :  Gaussian eliminatation
Calling Sequence : 
	calc_sigmas(USE num_par,rms,square, RETURN sigmas,correl_mtrx)
Parameter(s) :  	int num_par; number of parameters
			double rms; rms error = chisquared/(num_pts-num_par)
					where it is assumed that num_pts points
					are unit-weight points.  If num_pts
					= num_par, rms should be 0
			double square[]; square matrix (num_par x num_par
				elements) stored as a vector
			double sigmas[];  returned sigma's
			double correl_mtrx[]; returned correlation matrix,
				stored as a vector
Returned Value(s) :  0 - SUCCEED or FAIL (-1)
Routines Called :  solve_lin()
Bugs and Restrictions :  arrays are not destroyed, but it is assumed that rms
	has been calculated before entry.  Storage must be allocated for
	sigmas[] and correl_mtrx[] prior to calling this function
See also :  nlin.c
Revisions :  
		8/26/86 - development version
*/
/*FINISH*/

#include <stdio.h>
#include <math.h>
#include "nldef.h"
#define USE
#define RETURN

int
calc_sigmas(USE num_par,rms,square, RETURN sigmas,correl_mtrx)

int num_par;
double rms;
double square[];
double sigmas[];
double correl_mtrx[];
{

	static double a[MAXPAR][MAXPAR];
	static double cormat[MAXPAR][MAXPAR];
	static double dmtx[MAXPAR][MAXPAR];
	static double b[MAXPAR];

	int i,j,k;

/* check for legal number of parameters */

	if((num_par <=0)||(num_par > MAXPAR))
		return(-1);

/* copy square array into temporary storage */

	for(i=0;i<num_par;i++)
		for(j=0;j<num_par;j++)
			dmtx[i][j] = a[i][j] = square[i + j * num_par];

/* form correlation matrix */

	for(i=0;i<num_par;i++)
	{
		for(j=0;j<num_par;j++)
			b[j] = 0.0;
		b[i] = 1.0;

		if(solve_lin(num_par,a,b) != 1)
			return(-1);

		for(j=0;j<num_par;j++)
		{
			cormat[i][j] = b[j];
			for(k=0;k<num_par;k++)
				a[j][k] = dmtx[j][k];
		}
	}	

/* solve for the sigmas */

	for(i=0;i<num_par;i++)
		sigmas[i] = sqrt(cormat[i][i]);

/* normalize the correlation matrix */

	for(i=0;i<num_par;i++)
		for(j=0;j<num_par;j++)
			cormat[i][j] /= (sigmas[i] * sigmas[j]);

/* renormalize the sigma's - note that data are assumed to be unit weight! */

	for(i=0;i<num_par;i++)
		sigmas[i] *= rms;

/* convert correlation matrix to a one-dimensional array */

	for(i=0;i<num_par;i++)
		for(j=0;j<num_par;j++)
			correl_mtrx[i + j*num_par] = cormat[i][j];	

	return(0);
}
