/*  dmean.c   */
/*  jjk   */
/*  03-01-90   */

/*  calculates mean and sigma of a given array, passed by pointer to
	first member and number of doubles.  Optional arguement to
	recalculate mean and sigma after rejecting points nsigma away.
	This is not as rigorous as other algorithms (see, e.g.,
	Chauvenet's Criterion from Taylor's AN INTRODUCTION TO ERROR
	ANALYSIS p.142, but should be close given a "reasonable"
	nsigma.
	Remember that ptr has to be a ptr to an array of doubles!
*/

#include <math.h>
#include "mean.h"

struct MEAN dmean(ptr, nobj, nsigma)
    int nobj;
    double nsigma, *ptr;
{
    register i;
    int flag=1;
    double sum=0., sqr=0.;
    struct MEAN avg;


    if(nobj<0 || nsigma<0)
	error("DMEAN:  nobj or nsigma <0\n");
    avg.nsig=nsigma;
    avg.nreject=0;
    for(i=0; i<nobj; i++)
    {
	sum += *(ptr+i);
	sqr += *(ptr+i)* *(ptr+i);
    }
    avg.mean=sum/(double)nobj;
    avg.sigma=sqrt(sqr/(double)(nobj - 1) - (double)nobj/
	(double)(nobj - 1)*avg.mean*avg.mean);
    sum=sqr=0.;
    if(nsigma)
    {
    	for(i=0; i<nobj; i++)
	    if(fabs(fabs(*(ptr+i)) - fabs(avg.mean)) > nsigma*avg.sigma)
		avg.nreject++;
	    else
	    {
		sum += *(ptr+i);
		sqr += *(ptr+i)* *(ptr+i);
	    }
	avg.mean=sum/(double)(nobj - avg.nreject);
        avg.sigma=sqrt(sqr/(double)(nobj - avg.nreject - 1) - (double)
	    (nobj - avg.nreject)/(double)(nobj - avg.nreject - 1)
	    *avg.mean*avg.mean);
    }
    return(avg);
}
