/*  jjk   */
/*  fmean.c   */
/*  03-01-90   */

/*  calculates mean and sigma of a given array, passed by pointer to
	first member and number of floats.  Optional arguement to
	recalculate mean and sigma after rejecting points nsigma away.
	This is not as rigorous as other algorithms (see, e.g.,
	Chauvenet's Criterion from Taylor's AN INTRODUCTION TO ERROR
	ANALYSIS p.142, but should be close given a "reasonable"
	nsigma).
	Remember that ptr has to be a ptr to an array of floats!
*/
/*   05-24-90:  Added median and mode.  MODE IS ONLY AN APPROXIMATION
		(see note below).  A more precise value of mode
		could be obtained by integerizing the ptr2 array
		and finding the true mode.  (This would work only 
		if the integer part was a good approximation to the
		value, but typically true with image data). */
/*   07-17-90:  added sum and origsum to mean.h  */
/*   12-20-90:  still to be done:  should check to see that sqrt is
			defined if nreject is large (2nd sqrt)  */
/*   12-27-91:  MAJOR BUG:  I made an unwarrented assumption such
			that any use of this routine before this
			date will have the value of the
			uncertainty (sigma) be too large.  It can
			be an order of magnitude too large or
			perhaps only a factor of two.  NOTE:  only
			the sigma is affected, and not the mean.  */

#include <stdio.h>
#include <math.h>
/*#include <memory.h>  */
#include "mean.h"

struct MEAN fmean(ptr, nobj, nsigma)
    int nobj;
    float *ptr;
    double nsigma;
{
    register i;
    float *ptr2;
    float fcompar();
    struct MEAN avg;


/*printf("%g %d %f\n", *ptr, nobj, nsigma);  */
    avg.sigma=avg.sum=0.;
    if(nobj<0 || nsigma<0)
	error("FMEAN:  nobj or nsigma <0\n");
    if((ptr2=(float *)malloc(nobj*sizeof(float)))==NULL)
	error("FMEAN:  MALLOC\n");
    avg.nsig=nsigma;
    avg.nreject=0;
    for(i=0; i<nobj; i++)
	avg.sum += *(ptr+i);
    avg.mean=avg.sum/(double)nobj;
    for(i=0; i<nobj; i++)
	avg.sigma+=(*(ptr+i) - avg.mean)*(*(ptr+i) - avg.mean);
    avg.sigma=sqrt(avg.sigma/(double)(nobj - 1));
/*printf("MEAN:  %g %g\n", avg.mean, avg.sigma);  */
    if(nsigma)
    {
	avg.origmean=avg.mean;
    	avg.origsigma=avg.sigma;
    	avg.origsum=avg.sum;
    	avg.sigma=avg.sum=0.;
    	for(i=0; i<nobj; i++)
	    if(fabs(fabs(*(ptr+i)) - fabs(avg.mean)) > nsigma*avg.origsigma)
		avg.nreject++;
	    else
	    {
		*(ptr2+i)= *(ptr+i);
		avg.sum += *(ptr+i);
	    }
	avg.mean=avg.sum/(double)(nobj - avg.nreject);
    	for(i=0; i<nobj; i++)
	    if(fabs(fabs(*(ptr+i)) - fabs(avg.mean)) <= nsigma*avg.origsigma)
		avg.sigma+=(*(ptr+i) - avg.mean)*(*(ptr+i) - avg.mean);
        avg.sigma=sqrt(avg.sigma/(double)(nobj - avg.nreject - 1));
    }
    else
    	ptr2=(float *)memcpy((char *)ptr2, (char *)ptr, 
	    sizeof(float)*nobj);
    /*
     *   The following computes the median and an approximation 
     *   to the mode (see CRC math tables).  
     */
    qsort((char *)ptr2, nobj - avg.nreject, sizeof(float), fcompar);
    if((nobj - avg.nreject)%2)
        avg.median= *(ptr2 + (nobj - avg.nreject)/2);
    else
	avg.median=(*(ptr2 + (nobj - avg.nreject)/2) + *(ptr2 + 
	    ((nobj - avg.nreject) - 1)/2))/2.;
    avg.mode=3.*avg.median - 2.*avg.mean;
    /*
     *  Diagnostic prints
     */
/*
    printf("mean=%f +/- sigma=%f with %d rejected at %f sigma\n", 
	avg.mean, avg.sigma, avg.nreject, avg.nsig);
    printf("orig=%f +/- sigma=%f\n", avg.origmean, avg.origsigma);
    printf("median=%f\n", avg.median);
    printf("mode=%f\n", avg.mode);
    for(i=0; i<nobj - avg.nreject; i++)
	printf("%f\n", *(ptr2 + i));
*/
    free(ptr2);
    return(avg);
}
