/*   numderiv.c   */
/*   jjk   */
/*   10-23-91  */

/*   calculates numerical derivative for a vector of numbers.
	Assumes that the values are equally spaced in x and finds the
	derivative wrt x.  Assumes p=0 (notation of Abramavitz and
	Stegun (sp?) p. 882) and h=1 (although I will make them
	arguements). 
	order is number of points
	h is spacing:  h=x[i+1] - x[i]
	p is position of derivative:  f'p=f'(x0 + ph)
	*/
/*   ptr is a ptr to an array of floats   */

#include <stdio.h>
#include <math.h>
#include "deriv.h"

struct DERIV numderiv(ptr, sig, n, order, h, p)
int n, order;
float *ptr, *sig;
double h, p;
{
    register i;
    double a, b, c, d, e;
    struct DERIV funk;

/*printf("%f %f %d %d %f %f\n", *ptr, *sig, n, order, h, p);  */
    switch(order)
    {
	case 3:
	    a=p - .5;
	    b=-2.*p;
	    c=p + .5;
	    break;
	case 4:
            a=(-3.*p*p + 6.*p - 2.)/6.;
	    b=(3.*p*p - 4.*p - 1.)/2.;
	    c=(-3.*p*p + 2.*p + 2.)/2.;
	    d=(3.*p*p - 1.)/6.;
	    break;
	case 5:
	    a=(((2.*p - 3.)*p - 1.)*p + 1.)/12.;
	    b=(((-4.*p + 3.)*p + 8.)*p - 4.)/6.;
	    c=p*(2.*p*p - 5.)/2.;
	    d=(((-4.*p - 3.)*p + 8.)*p + 4.)/6.;
	    e=(((2.*p + 3.)*p - 1.)*p - 1.)/12;
	    break;
	case '?':
	    error("NUMDERIV:  order must be 3, 4, or 5\n");
	default:
	    error("NUMDERIV:  impossible\n");
    }
    if((funk.fprime=(float *)malloc(n*sizeof(float)))==NULL)
	error("numderiv:  MALLOC\n");
    if((funk.sigfprime=(float *)malloc(n*sizeof(float)))==NULL)
	error("numderiv:  MALLOC\n");
    for(i=0; i<(order-1)/2; i++)
	funk.fprime[i]=funk.sigfprime[i]=0.;
    for(i=(order-1)/2; i<n-order/2; i++)
    {
	switch(order)
	{
	    case 3:
		funk.fprime[i]=(a*ptr[i-1] + b*ptr[i] + c*ptr[i+1])/h;
		funk.sigfprime[i]=sqrt(a*a*sig[i-1]*sig[i-1] +
	    	    b*b*sig[i]*sig[i] + c*c*sig[i+1]*sig[i+1])/h;
		break;
	    case 4:
		funk.fprime[i]=(a*ptr[i-1] + b*ptr[i] + c*ptr[i+1] +
	    	    d*ptr[i+2])/h;
		funk.sigfprime[i]=sqrt((a*a*sig[i-1]*sig[i-1] +
	    	    b*b*sig[i]*sig[i] + c*c*sig[i+1]*sig[i+1] +
	    	    d*d*sig[i+2]*sig[i+2]))/h;
		break;
	    case 5:
		funk.fprime[i]=(a*ptr[i-2] + b*ptr[i-1] + c*ptr[i] +
	    	    d*ptr[i+1] + e*ptr[i+2])/h;
		funk.sigfprime[i]=sqrt(a*a*sig[i-2]*sig[i-2] +
	    	    b*b*sig[i-1]*sig[i-1] + c*c*sig[i]*sig[i] +
	    	    d*d*sig[i+1]*sig[i+1] + e*e*sig[i+2]*sig[i+2])/h;
		break;
	}
    }
    for(i=n-order/2; i<n; i++)
	funk.fprime[i]=funk.sigfprime[i]=0.;
    return(funk);
}
