/*START*/
/*TITLE solve_lin() */
/*
Source File :  /usr/local/utility/src/solve_lin.c
Author(s) : rgf
Latest Revision Date :  8/27/86
Description :  solve a system of linear equations
Algorithm :  identical to routine "clvlnr" in nlin.c except that the square
	matrix and the vector are passed as arguments
Calling Sequence :  nerr = solve_lin(numpar,a,b);
Parameter(s) :  	int	numpar - # param to fit
			double  a[][MAXPAR] - square array (modified)
			double  b[] - vector (modified)
Returned Value(s) :  nerr = 1 => ok  nerr = 2 ==> singular matrix
Routines Called :  math routines
Bugs and Restrictions :  no check for numpar > MAXPAR - this should be
	done BEFORE this routine is called!
See also :  /usr/local/utility/src/nlin.c
Revisions :  
		8/27/86 - development version - rgf
*/
/*FINISH*/
#include <stdio.h>
#include <math.h>
#include "nldef.h"

/*	solve_lin returns an integer error flag, with the following meaning: 
 *	nerr =	1	ok                                             
 *		2	singular matrix when computing new parameters 
 */

int
solve_lin(numpar,a,b)
int numpar;
double a[][MAXPAR];
double b[];
{

	int i, j, k;
	int irow, icolum, icl, irl, l;
	int iflag = 1;
	double pivsiz = 0;
	double ipivot[MAXPAR];
	double amax, aa, swap, pivot, t;
	double sqrt(),fabs(),pow();

	for(j = 0; j < numpar; j++)
		ipivot[j] = 0;
	for(i = 0; i < numpar; i++)
	{
		for(j = 0; j < numpar; j++)
			pivsiz = pivsiz + a[i][j] * a[i][j];
	}
	pivsiz = sqrt(pivsiz) * pow(10,-16)/(numpar*numpar);
	for(i = 0; i < numpar; i++)
	{
		amax = 0;
		for(j = 0;j < numpar; j++)
		{
			if(ipivot[j] != 1)
			{
				for(k = 0; k < numpar; k++)
				{
					if(ipivot[k] != 1)
					{	
						aa = fabs(a[j][k]);
						if(amax < aa)
						{
							irow = j;
							icolum = k;
							amax = aa;
						}
					}
				}
			}
		}
		ipivot[icolum]++;
		if(irow != icolum)
		{
			irl = irow;
			icl = icolum;
			for(l = 0; l < numpar; l++)
			{
				swap = a[icl][l];
				a[icl][l] = a[irl][l];
				a[irl][l] = swap;
			}
			swap = b[irow];
			b[irow] = b[icolum];
			b[icolum] = swap;
		}
		pivot = a[icolum][icolum];
		if(fabs(pivot) < pivsiz)
		{
			iflag = 2;
			return(iflag);
		}
		a[icolum][icolum] = 1;
		icl = icolum - numpar;
		for(l = 0; l < numpar; l++)
			a[icolum][l] = a[icolum][l]/pivot;
		b[icolum] = b[icolum]/pivot;
		for(j = 0;j < numpar; j++)
		{
			if(j != icolum)
			{
				t = a[j][icolum];
				a[j][icolum] = 0;
				for(k = 0; k < numpar; k++)
					a[j][k] = a[j][k] - a[icolum][k] * t;
			b[j] = b[j] - b[icolum] * t;
			}
		}
	}
	return(iflag);
}
