/*---------------------------------------------------------------
           FILE: bilin_interp.c
      $Revision: 2.0 $
         $State: Exp $
  Last submitted $Date: 93/03/30 17:59:53 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	bilin_interp.c,v $
 * Revision 2.0  93/03/30  17:59:53  jh
 * Actual first release version.
 * 
 * Revision 1.2  93/03/30  14:00:07  jh
 * General code cleanup, testing, and added features.
 * First release version.
 * 
 * Revision 1.1  92/12/15  13:03:30  jh
 * Initial revision
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/bilin_interp.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/bilin_interp.c,v 2.0 93/03/30 17:59:53 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
bilin_interp.c = bilinear interpolation function

External object files or libraries:
-lm
---------------------------------------------------------------*/
/* inclusions */

#include <math.h>
#include "bilin_interp.h"

/*---------------------bilin_interp------------------------------
Effect: Does bilinear interpolation to find z[y][x]; returns
        value or DBL_MAX if x or y are out of bounds.

Requires: z: pointer to a Doub2 which tabulates the function
             being interpolated.
	  x, y: doubles which are the position at which to
	        interpolate the function.

Modifies: nothing.

Implementation notes: The order of arguments is such that the
                      arguments appear as close as possible to C
		      array notation: z[y][x].
---------------------------------------------------------------*/
double
bilin_interp(
	     Doub2  *z,
	     double  y,
	     double  x
){
    int a = floor(x);
    int b = ceil(x);
    int c = floor(y);
    int d = ceil(y);

    double bx = (double)b - x;
    double xa = x - (double)a;
    double dy = (double)d - y;
    double yc = y - (double)c;

    /* if the coords are exactly integers, floor and ceil are equal */
    if ( xa == 0. ) {
	bx = 1.;
    }
    if ( yc == 0. ) {
	dy = 1.;
    }

    if ( x < 0. ||
	y < 0. ||
	x > (double) z->nj - 1 ||
	y > (double) z->ni - 1 ){
	return DBL_MAX;
    }

    return dy * ( bx * z->data[c][a] + xa * z->data[c][b])
	 + yc * ( bx * z->data[d][a] + xa * z->data[d][b]);
}
