/*---------------------------------------------------------------
           FILE: irafio.c
      $Revision: 2.0 $
         $State: Exp $
  Last submitted $Date: 93/03/30 18:01:57 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	irafio.c,v $
 * Revision 2.0  93/03/30  18:01:57  jh
 * Mosaic functions moved to irafmosdat.
 * Actual first release version.
 * 
 * Revision 1.2  93/03/30  13:54:19  jh
 * General code cleanup, testing, and added features.
 * First release version.
 * 
 * Revision 1.1  92/12/15  13:04:25  jh
 * Initial revision
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/irafio.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/irafio.c,v 2.0 93/03/30 18:01:57 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
irafio.c = some IRAF image io functions

External object files or libraries:

-ljjkimc
-ljjkutil

lib$ = the lib directory under iraf
hlib$ = the unix/hlib directory under iraf
lib$/libimfort.a
lib$/libsys.a
lib$/libvops.a
hlib$/libos.a

Depending on operating system, other externals may be necessary
for IRAF imfort.
---------------------------------------------------------------*/
/* definitions */

#define JJKIMCFNMAX 120

/* inclusions */

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <imiraf.h>
#include "memory.h"
#include "irafio.h"

/* data types */

typedef struct IMAGE Image;

/*----------------------iraf_read_float_image--------------------
Effect: Reads an IRAF OIF format float image into a Doub2 * and
        returns a pointer to it, or NULL on error.

Requires: filename: name of image.

Modifies: nothing.

Implementation notes: Free the Doub2 with free_doub2().
---------------------------------------------------------------*/
Doub2 *
iraf_read_float_image(char *filename){
    size_t  i;
    size_t  j;
    Image  in;
    int    ind;
    Doub2 *out;

    ind = imropen(filename);
    if(ind == -1){
	return NULL;
    }

    in = openfimg(ind, in);
    if ( in.pixels == NULL){
	return NULL;
    }
    imclose(ind);

    out = alloc_doub2(in.rows, in.cols);
    if (!out){
	return NULL;
    }    

    /* convert floating->double precision */
    for ( i = 0 ; i < in.rows ; i++ ){
	for ( j = 0 ; j < in.cols ; j++ ){
	    out->data[i][j] = (double)*(in.pixels + i * in.cols + j);
	}
    }

    return out;
}

/*----------------------iraf_write_float_image-------------------
Effect: Writes an IRAF OIF format float image pointed to by a
        Doub2 * and returns SUCCESS or FAILURE.

Requires: filename: name of image file to be created.

          out: pointer to Doub2 containing image.

Modifies: file named by filename.

Implementation notes:
---------------------------------------------------------------*/
int
iraf_write_float_image(
		       char *filename,
		       Doub2 *out
){
    size_t  i;
    size_t  j;
    Image   i_out;
    int     outd;
    int     wrret;

    /* initialize Image */
    strncpy(i_out.file, filename, JJKIMCFNMAX);
    i_out.file[JJKIMCFNMAX] = '\0';
    i_out.cols    = out->nj;
    i_out.rows    = out->ni;
    i_out.pixtype = FLOAT;

    i_out.pixels = (float *)malloc(i_out.rows * i_out.cols * sizeof(float));
    if (!i_out.pixels){
	return FAILURE;
    }

    i_out.box.x1 = 0;
    i_out.box.x2 = 0;
    i_out.box.y1 = 0;
    i_out.box.y2 = 0;

    /* open file */
    outd = imrwopen(i_out);
    if(outd == -1){
	return FAILURE;
    }

    /* convert double precision->floating */
    for ( i = 0 ; i < i_out.rows ; i++ ){
	for ( j = 0 ; j < i_out.cols ; j++ ){
	    *(i_out.pixels + i * i_out.cols + j) = (float) out->data[i][j];
	}
    }

    /* write file */
    wrret = imps2r(outd, i_out);

    /* check errors and clean up */

    free(i_out.pixels);

    if ( 0 != wrret ){
	imclose(outd);
	return FAILURE;
    }
    
    imclose(outd);

    return SUCCESS;
}
