/*---------------------------------------------------------------
           FILE: limbctr.c
      $Revision: 1.1 $
         $State: Exp $
  Last submitted $Date: 1995/04/14 01:54:58 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log: limbctr.c,v $
 * Revision 1.1  1995/04/14  01:54:58  jh
 * Initial revision
 *

  $Source: /home/jh/ast/jiggle/limbctr/RCS/limbctr.c,v $

  Copyright (c) 1993 Joseph Harrington.  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /home/jh/ast/jiggle/limbctr/RCS/limbctr.c,v 1.1 1995/04/14 01:54:58 jh Exp jh $";
static char copyright[] =
    "Copyright (c) 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
limbctr.c = program to fit an ellipse to a planetary limb

External object files or libraries:
bilin_interp.c
irafio.c
memory.c
simplex.c
-ljjkimc
-ljjkutil
-lm
lib$ = the lib directory under iraf
hlib$ = the unix/hlib directory under iraf
lib$/libimfort.a
lib$/libsys.a
lib$/libvops.a
hlib$/libos.a

Depending on operating system, other externals may be necessary
for IRAF imfort.
---------------------------------------------------------------*/
/* definitions */
#define SUCCESS 1
#define FAILURE 0
#define TRUE  1
#define FALSE 0

#define Streq(s1, s2) ( strcmp( s1, s2 ) == 0 ) /* These strings the same? */
#define Min(a, b) ((a) < (b) ? (a) : (b))

#define SIDE 5
#define RESTART_SIDE 1

/* inclusions */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "planetmap.h"
#include "limbeval.h"
#include "memory.h"
#include "simplex.h"
#include "irafio.h"

/* data types */

typedef struct {
    double ftol;	/* parameters for minimizer, see */
    double flip;	/* simplex.c and Numerical Recipes 10.4 */
    double extend;
    double contract;
    double funcmax;
    int    diag;        /* print diagnostic info when done? */
} Data;

/* prototypes */
Data *
create_data(void);

int
interp_command_line_args( 
    int           argc,
    char         *argv[],
    Data         *dat,
    Limbevalpars *limb
);

void 
usage(char *progname);

void
print_diag(
	   Data          *dat,
	   Limbevalpars  *limb,
	   Simplex_param *simp
);

/*---------------------------------------------------------------
Effect:
Requires:
Modifies:
Implementation notes:
---------------------------------------------------------------*/
int
main(int argc, char *argv[]){
    Data          *dat;
    Limbevalpars    *limb;
    Simplex_param *simp;

    dat = create_data();
    if (!dat){
	exit(1);
    }

    limb = create_limbevalpars();
    if (!limb){
	exit(1);
    }

    /* parse command line, allocating space as needed */
    if (!interp_command_line_args(argc, argv, dat, limb)){
	exit(1);
    }

    /* Initialize Simplex */
    simp = simplex_create( limb->fit_npars );

    LE_xc(simp->p[0]) = limb->nom_xc == DBL_MAX
			 ? limb->image->nj / 2.
			 : limb->nom_xc;
    LE_yc(simp->p[0]) = limb->nom_yc == DBL_MAX
			 ? limb->image->ni / 2.
			 : limb->nom_yc;
    LE_re(simp->p[0]) = limb->nom_re;

    if(limb->fit_lt){
	LE_lt(simp->p[0]) = 0.;
    }

    (void)simplex_init_param(simp, limbeval, limb,
			     dat->ftol, dat->flip, dat->extend,
			     dat->contract, dat->funcmax);

    (void)simplex_init_point(simp, SIDE);

    /* Call Minimizer */
    (void)simplex(simp);

    /* restart */
    simp->funcmax *= 2;
    (void)simplex_init_point(simp, RESTART_SIDE);
    (void)simplex(simp);

    /* write output */
    printf("%.2lf %.2lf %.2lf",
	   LE_xc(simp->p[0]) + 1, /* add one for FITS/IRAF coordinates */
	   LE_yc(simp->p[0]) + 1, /* ditto */
	   LE_re(simp->p[0]));
    if(limb->fit_lt){
	printf(" %.2lf\n", LE_lt(simp->p[0]));
    }else{
	printf("\n");
    }

    /* print diagnostic info */
    if ( dat->diag ){
        print_diag(dat, limb, simp);
    }

    /* free memory */
    simplex_free(simp);
    free_limbevalpars(limb);
    free(dat);

    return 0;
}

/*------------------create_data----------------------------------
Effect: Allocates a Data structure and returns a pointer to it.
        Prints an error message and returns NULL on error.

Requires: nothing.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
Data *
create_data(void){
    Data *dat;
    
    dat = (Data *)malloc(sizeof(*dat));
    if(!dat){
	fprintf(stderr, "create_data: unable to allocate %u bytes\n",
		sizeof(*dat));
	return NULL;
    }

    /* defaults */
    dat->ftol     = 1e-7;
    dat->flip     = 1.;
    dat->extend   = 2.;
    dat->contract = 0.5;
    dat->funcmax  = 50000;
    dat->diag     = FALSE;

    return dat;
}

/*---------------------interp_command_line_args------------------
Effect: Parses command line and stuffs dat struct appropriately.
        Returns SUCCESS if parsed correctly and FAILURE if not.

Requires: argc and argv: as from main()
          dat: pointer to dat structure of stuff.

Modifies: *dat.

Implementation notes:
---------------------------------------------------------------*/
int
interp_command_line_args( 
    int           argc,
    char         *argv[],
    Data         *dat,
    Limbevalpars *limb
){
    int i; /* argument loop counter */
    int j = 0; /* positional argument counter */
    int minpos = 1; /* minimal number of positional arguments */
    int maxpos = 1; /* maximum number of positional arguments */
    char **posv;

    double rot;	/* rotation angle */

    posv = (char **)malloc(argc * sizeof(*posv));
    if (!posv){
		fprintf(stderr,
		     "interp_command_line_args: unable to allocate %u bytes.\n",
		argc * sizeof(*posv));
	return FAILURE;
    }

    for ( i = 1 ; i < argc ; i++ ) {
	if     (   Streq( argv[i], "-re"     )
		|| Streq( argv[i], "-radius" )
		|| Streq( argv[i], "-r"      ) ) {
	    limb->nom_re = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-x"  ) ) {
	    limb->nom_xc = atof(argv[++i]) - 1.; /* for FITS/IRAF coords */
	}
	else if(   Streq( argv[i], "-y"  ) ) {
	    limb->nom_yc = atof(argv[++i]) - 1.; /* for FITS/IRAF coords */
	}
	else if(   Streq( argv[i], "-maxdre" )
		|| Streq( argv[i], "-mdr"    ) ) {
	    limb->delt_re = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-maxlt" ) 
	        || Streq( argv[i], "-lt"    ) ) {
	    limb->delt_lt   = atof(argv[++i]);
	    limb->fit_lt    = TRUE;
	    limb->fit_npars = LE_LTNP;
	}
	else if(   Streq( argv[i], "-oblateness" )
		|| Streq( argv[i], "-ob"         ) ) {
	    limb->ellip = 1. - atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-pa" ) ) {
	    rot = PMradians(atof(argv[++i]));
	    limb->rc = cos(-rot);
	    limb->rs = sin(-rot);
	}
	else if(   Streq( argv[i], "-npoint" )
		|| Streq( argv[i], "-n"      ) ) {
	    limb->npoint = atoi(argv[++i]);
	}
	else if(   Streq( argv[i], "-minfrac" )
		|| Streq( argv[i], "-mf"      ) ) {
	    limb->minfrac = atof(argv[++i]);
	}
	else if( Streq( argv[i], "-ftol" ) ) {
	    dat->ftol = atof(argv[++i]);
	}
	else if( Streq( argv[i], "-flip" ) ) {
	    dat->flip = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-extend" )
		|| Streq( argv[i], "-ex"     ) ) {
	    dat->extend = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-contract" )
		|| Streq( argv[i], "-ct"       ) ) {
	    dat->contract = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-funcmax" )
		|| Streq( argv[i], "-fm"      ) ) {
	    dat->funcmax = atoi(argv[++i]);
	}
	else if(   Streq( argv[i], "-diag" )
		|| Streq( argv[i], "-d"    ) ) {
	    dat->diag = TRUE;
	}
	else if( Streq( argv[i], "-h" ) ||
		Streq( argv[i], "-help" ) ) {
	    usage(argv[0]);
	    return FAILURE;
  	}
	else { /* it's positional */
	    posv[j++] = argv[i];
	}
    }

    /* do stuff with positional arguments */
    if ( j < minpos || j > maxpos ) {
	fprintf(stderr, 
		"%s: have %d positional argument%s, need %d\n",
		argv[0], j, ( j == 1 ? "" : "s" ), minpos);
	usage(argv[0]);
	return FAILURE;
    }
    i = 0;
    strncpy( limb->imagename, posv[i++], FILENAME_MAX);

    /* fill in limbevalpars (does this really belong here?) */
    limb->image = iraf_read_float_image(limb->imagename);
    if (!limb->image){
	return FAILURE;
    }
    if ( limb->nom_xc > .99 * DBL_MAX ){
	limb->nom_xc = (double)limb->image->nj/2.;
    }
    if ( limb->nom_yc > .99 * DBL_MAX ){
	limb->nom_yc = (double)limb->image->ni/2.;
    }
    if ( limb->nom_re > .99 * DBL_MAX ){
	limb->nom_re = Min(limb->nom_xc, limb->nom_yc)/3.;
    }
    if ( limb->delt_re > .99 * DBL_MAX ){
	limb->delt_re = limb->nom_re/10.;
    }

    return SUCCESS;
}

/*------------------------usage----------------------------------
Effect: Prints the program usage.

Requires: progname: pointer to character string to print.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
void 
usage(char *progname)
{
    fprintf(stderr, "%s: Fit an ellipse to a planet in an IRAF image.\n",
	    progname);
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "%s image <options>\n", progname);
    fprintf(stderr, "\timage is the name of the input image\n");

    fprintf(stderr, "\toptions:\n");

    fprintf(stderr,
      "\t-re (-radius, -r) double: approximate equatorial radius (pixels),\n");
    fprintf(stderr, "\t\t\tdefault 1/3 of smaller image dimension.\n");

    fprintf(stderr, "\t-x double: X coordinate of center of planet,\n");
    fprintf(stderr, "\t\t\tdefault half of X image dimension.\n");

    fprintf(stderr, "\t-y double: Y coordinate of center of planet,\n");
    fprintf(stderr, "\t\t\tdefault half of Y image dimension.\n");

    fprintf(stderr,
	    "\t-maxdre (-mdr) double: max deviation in radius (pixels),\n");
    fprintf(stderr, "\t\t\tdefault: 1/4 of nominal radius.\n");

    fprintf(stderr,
	    "\t-maxlt (-lt) double: max limb thickness parameter (pixels),\n");
    fprintf(stderr, "\t\t\tdefault: not fit.\n");

    fprintf(stderr,
	    "\t-oblateness (-ob) double: planetary oblateness (Re - Rp)/Re,\n");
    fprintf(stderr, "\t\t\tdefault: 0.\n");

    fprintf(stderr, "\t-pa double: polar position angle (degrees),\n");
    fprintf(stderr, "\t\t\tdefault: 0.\n");

    fprintf(stderr,
	 "\t-npoint (-n) integer: number of ellipse points to evaluate,\n");
    fprintf(stderr, "\t\t\tdefault: 360.\n");

    fprintf(stderr,
	 "\t-minfrac (-mf) double: minimum fraction of ellipse on image,\n");
    fprintf(stderr, "\t\t\tdefault: 1.\n");

    fprintf(stderr,
      "\t-ftol double: fractional tolerence for minimization convergence,\n");
    fprintf(stderr, "\t\t\tdefault 1e-7\n");

    fprintf(stderr,
      "\t-flip double: flip parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault 1\n");

    fprintf(stderr,
     "\t-extend (-ex) double: extend parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault 2\n");

    fprintf(stderr,
 "\t-contract (-ct) double: contract parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault .5\n");

    fprintf(stderr,
"\t-funcmax (-fm) unsigned integer: maximum number of function evaluations,\n");
    fprintf(stderr, "\t\t\tdefault 50000\n");

    fprintf(stderr, "\t-diag (-d): print diagnostic information during run.\n");
    fprintf(stderr, "\t-help (-h): print this message and quit.\n");

    return;
}

/*----------------------print_diag-------------------------------
Effect: prints diagnostic extracted from arguments on stderr.

Requires: dat: pointer to Data.
          limb: pointer to Limbevalpars.
	  simp: pointer to Simplex_param.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
void
print_diag(
	   Data          *dat,
	   Limbevalpars  *limb,
	   Simplex_param *simp
){
    fprintf(stderr, "function evals: %u\n", simp->nfunc);

    return;
}
