/*---------------------------------------------------------------
           FILE: planetmap.h
      $Revision: 1.1 $
         $State: Exp $
  Last submitted $Date: 1995/04/14 01:57:15 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log: planetmap.h,v $
 * Revision 1.1  1995/04/14  01:57:15  jh
 * Initial revision
 *

  $Source: /home/jh/ast/jiggle/limbctr/RCS/planetmap.h,v $

  Copyright (c) 1993 Joseph Harrington.  All Rights reserved.
---------------------------------------------------------------*/
#ifndef PLANETMAP_PROTECT
#ifndef LINT
static char rcsidplanetmaph[] = "$Header: /home/jh/ast/jiggle/limbctr/RCS/planetmap.h,v 1.1 1995/04/14 01:57:15 jh Exp jh $";
static char copyrightplanetmaph[] =
    "Copyright (c) 1993 Joseph Harrington.  All Rights reserved.";
#endif
#endif

/*---------------------------------------------------------------
planetmap.h = include file for planetary mapping

---------------------------------------------------------------*/
#ifndef PLANETMAP_PROTECT
#define PLANETMAP_PROTECT

/* inclusions */
#include <math.h>

/* definitions */
#define PI ((double)3.14159265358979323846264338328)

/* convert between angle systems */
#define PMradians(theta) ((theta)*PI/180.)
#define PMdegrees(theta) ((theta)*180./PI)

/* minimal positive or negative value of angle */
#define PMradmod(theta) (fmod((theta), 2.*PI))
#define PMdegmod(theta) (fmod((theta), 360.))

/* minimal positive value from minimal positive or negative value */
#define PMradsign(theta) ( (theta) >= 0. ? (theta) : (theta) + 2.*PI )
#define PMdegsign(theta) ( (theta) >= 0. ? (theta) : (theta) + 360.  )

/* minimal positive value of angle */
#define PMradrng(theta) PMradsign(PMradmod((theta)))
#define PMdegrng(theta) PMdegsign(PMdegmod((theta)))

/* planetocentric - planetodetic conversion */
/* ellip is semiminor/semimajor axis ratio (e.g., Rp/Re) */
/* all angles are radians in range [0,2*PI) */

#define PMpcfpd(detic, ellip) \
    (PMradrng( ( ((detic) > PI/2.) && ((detic) <= 3.*PI/2.) \
		? PI : (double)0.) \
	      + atan( tan((detic)) * (ellip) * (ellip) ) ))

#define PMpdfpc(centric, ellip) \
    (PMradrng( ( ((centric) > PI/2.) && ((centric) <= 3.*PI/2.) \
		? PI : (double)0.) \
	      + atan( tan((centric)) / ((ellip) * (ellip)) ) ))

/* Planetocentric (ellipse central) - circular parameterization conversion */
#define PMpcfca(circang, ellip) \
    (PMradrng( ( ((circang) > PI/2.) && ((circang) <= 3.*PI/2.) \
		? PI : (double)0.) \
	      + atan( tan((circang)) * (ellip) ) ))

#define PMcafpc(centric, ellip) \
    (PMradrng( ( ((centric) > PI/2.) && ((centric) <= 3.*PI/2.) \
		? PI : (double)0.) \
	      + atan( tan((centric)) / (ellip) ) ))

/* Planetodetic - circular parameterization conversion */
#define PMpdfca(circang, ellip) PMcafpc((circang), (ellip))
#define PMcafpd(detic,   ellip) PMpcfca((detic),   (ellip))

/* data types */

/* prototypes */

#endif
